using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using Core;
using Core.Mapping;
using Core.Graphics;
using Client.World;
using System.Collections;


namespace Client
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class HxHClient : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        LocalWorld LocalWorld;
        string WorldDirectory;

        public HxHClient()
        {
            graphics = new GraphicsDeviceManager(this);
            //Set content manager so that other game components can use it.
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            //Load World Directory Path
            WorldDirectory = @"C:\HxHWorld\27\";
            //Setup Map Engine
            LocalWorld = new LocalWorld(this, WorldDirectory);
            this.Components.Add(LocalWorld);
            this.IsMouseVisible = true;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            ProcessKeyboard();
            ProcessMouse();

            base.Update(gameTime);
        }

        private void ProcessMouse()
        {
            MouseState buttons = Mouse.GetState();

            if (buttons.LeftButton == ButtonState.Pressed || buttons.RightButton == ButtonState.Pressed)
            {
                LocalWorld.MouseDown(buttons);
            }
        }

        private void ProcessKeyboard()
        {
            //Get KeyboardState
            KeyboardState keys = Keyboard.GetState();
            //If there are pressed Keys
            if (keys.GetPressedKeys().Length > 0)
            {
                LocalWorld.KeyDown(keys);
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            base.Draw(gameTime);
        }
    }
}
